package com.huifu.jh.core;

import com.alibaba.fastjson.JSON;
import com.huifu.jh.core.exception.GcbPayException;
import com.huifu.jh.core.model.CallbackMessage;
import com.huifu.jh.core.model.ParamCallbackMessage;
import com.huifu.jh.core.model.XmlCallbackMessageWrapper;
import com.huifu.jh.core.util.ServletUtils;
import com.huifu.jh.core.util.SignUtils;
import com.huifu.jh.core.util.XMLSecurityUtils;
import com.huifu.jh.core.util.XmlUtils;
import org.apache.commons.io.IOUtils;
import org.w3c.dom.Document;

import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.util.Map;

/**
 * @author fang.wei
 * @Description 异步回调解析器
 * @createTime 2021年10月18日 12:41:00
 */
public class DefaultGcbPayCallbackMessageParser implements GcbPayCallbackMessageParser {

    /**
     * 回调对象封装类默认后缀 如：xxxCallbackMessageWrapper
     */
    private static final String WRAPPER_TYPE_SUFFIX = "Wrapper";

    /**
     * 汇付国际公钥
     */
    private String gcbPublicKey;

    public DefaultGcbPayCallbackMessageParser(String gcbPublicKey) {
        this.gcbPublicKey = gcbPublicKey;
    }

    @Override
    public <T extends CallbackMessage> T parse(HttpServletRequest request, Class<T> clazz) throws Exception {
        if (ParamCallbackMessage.class.isAssignableFrom(clazz)) {
            return parseMessage(request, clazz);
        }

        return parseXmlMessage(request, clazz);
    }

    /**
     * 解析参数异步回调
     * @param request
     * @param clazz
     * @param <T>
     * @return
     */
    private <T extends CallbackMessage> T parseMessage(HttpServletRequest request, Class<T> clazz) throws GcbPayException {
        Map<String, String> data = ServletUtils.convertToMap(request);

        String jsonStr = JSON.toJSONString(data);
        T message = JSON.parseObject(jsonStr, clazz);

        ParamCallbackMessage paramMessage = (ParamCallbackMessage) message;
        paramMessage.setData(data);
        String sourceMsg = SignUtils.buildSignSourceMsg(paramMessage.getSignParamNames(), data);
        boolean verifyResult = SignUtils.verifyMessage(gcbPublicKey, sourceMsg, paramMessage.getSignMsg());
        if (!verifyResult) {
            throw new GcbPayException("verify callback message signature error");
        }
        return message;
    }

    /**
     * 解析XML异步回调
     * @param request
     * @param clazz
     * @param <T>
     * @return
     */
    private <T extends CallbackMessage> T parseXmlMessage(HttpServletRequest request, Class<T> clazz) throws Exception {
        InputStream is = request.getInputStream();

        try {
            String xmlMessage = IOUtils.toString(is);
            Document doc = XmlUtils.toDocument(xmlMessage);
            boolean verifyResult = XMLSecurityUtils.verifySignature(doc, gcbPublicKey);
            if (!verifyResult) {
                throw new GcbPayException("verify response signature failed");
            }

            // 获取封装类
            Class<T> wrapperClass = tryGetWrapperClass(clazz);
            if (wrapperClass != null) {
                return parseMessageByWrapper(xmlMessage, wrapperClass);
            }

            // 未定义封装类则直接转换
            T message = XmlUtils.toBean(xmlMessage, clazz);
            return message;
        } finally {
            IOUtils.closeQuietly(is);
        }
    }

    /**
     *
     * @param xmlMessage
     * @param wrapperClass
     * @param <T>
     * @return
     * @throws Exception
     */
    private <T extends CallbackMessage> T parseMessageByWrapper(String xmlMessage, Class<T> wrapperClass) throws Exception {
        XmlCallbackMessageWrapper<T> messageWrapper = (XmlCallbackMessageWrapper<T>) XmlUtils.toBean(xmlMessage, wrapperClass);
        return messageWrapper.toMessage();
    }

    /**
     * 尝试获取包装类
     * @param clazz
     * @param <T>
     * @return
     */
    private <T extends CallbackMessage> Class tryGetWrapperClass(Class<T> clazz) {
        try {
            String path = clazz.getPackage().getName() + ".wrapper.";
            String name = clazz.getSimpleName() + WRAPPER_TYPE_SUFFIX;
            String wrapperClassName = path + name;
            return Class.forName(wrapperClassName);
        } catch (ClassNotFoundException e) {
            // 未定义包装类直接返回null
            return null;
        }
    }

    public String getGcbPublicKey() {
        return gcbPublicKey;
    }

    public void setGcbPublicKey(String gcbPublicKey) {
        this.gcbPublicKey = gcbPublicKey;
    }
}