package com.huifu.jh.core.util;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import com.huifu.jh.core.exception.GcbPayException;
import org.apache.commons.codec.binary.Base64;

import java.security.KeyFactory;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

/**
 * 秘钥提取工具类
 */
public class SecretKeyUtils {

	/**
	 * 从pfx证书文件中获取私钥
	 * @param privateKeyFilePath
	 * @param alias
	 * @param password
	 * @return
	 */
	public static PrivateKey getPrivateKey(String privateKeyFilePath, String alias, String password) throws GcbPayException {
		InputStream is = null;
		try {
			KeyStore keyStore = KeyStore.getInstance("PKCS12");
			is = new BufferedInputStream(new FileInputStream(privateKeyFilePath));
			char[] keyPwd = password.toCharArray();
			keyStore.load(is, password.toCharArray());
			return (PrivateKey) keyStore.getKey(alias, keyPwd);
		} catch (Exception e) {
			throw new GcbPayException("convert privateKey error", e);
		} finally {
			if (is != null) {
				try {
					is.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	/**
	 * 从证书文件中获取公钥
	 * @param publicKeyFilePath
	 * @return
	 * @throws Exception
	 */
	public static PublicKey getPublicKey(String publicKeyFilePath) throws GcbPayException {
		InputStream is = null;
		try {
			CertificateFactory certificatefactory = CertificateFactory.getInstance("X.509");
			is = new FileInputStream(publicKeyFilePath);
			X509Certificate cert = (X509Certificate) certificatefactory.generateCertificate(is);
			return cert.getPublicKey();
		} catch (Exception e) {
			throw new GcbPayException("convert publicKey error", e);
		} finally {
			if (is != null) {
				try {
					is.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	/**
	 * 字符串私钥转换私钥
	 * @param privateKeyString
	 * @return
	 */
	public static PrivateKey transform2PrivateKey(String privateKeyString) throws GcbPayException {
		try {
			PKCS8EncodedKeySpec spec = new PKCS8EncodedKeySpec(Base64.decodeBase64(privateKeyString));
			KeyFactory keyFactory = KeyFactory.getInstance("RSA");
			return keyFactory.generatePrivate(spec);
		} catch (Exception e) {
			throw new GcbPayException("convert privateKey error", e);
		}
	}

	/**
	 * 字符串公钥转换公钥
	 * @param publicKeyString
	 * @return
	 * @throws GcbPayException
	 */
	public static PublicKey transform2PublicKey(String publicKeyString) throws GcbPayException {
		try {
			X509EncodedKeySpec spec = new X509EncodedKeySpec(Base64.decodeBase64(publicKeyString));
			KeyFactory keyFactory = KeyFactory.getInstance("RSA");
			return keyFactory.generatePublic(spec);
		} catch (Exception e) {
			throw new GcbPayException("convert publicKey error", e);
		}
	}

	/**
	 * 私钥证书转换成字符串私钥
	 * @param privateKeyFilePath
	 * @param alias
	 * @param password
	 * @return
	 * @throws GcbPayException
	 */
	public static String transform2PrivateKeyString(String privateKeyFilePath, String alias, String password) throws GcbPayException {
		PrivateKey privateKey = getPrivateKey(privateKeyFilePath, alias, password);
		return Base64.encodeBase64String(privateKey.getEncoded());
	}

	/**
	 * 公钥证书转换成字符串公钥
	 * @param publicKeyFilePath
	 * @return
	 * @throws GcbPayException
	 */
	public static String transform2PublicKeyString(String publicKeyFilePath) throws GcbPayException {
		PublicKey publicKey = getPublicKey(publicKeyFilePath);
		return Base64.encodeBase64String(publicKey.getEncoded());
	}

	public static void main(String[] args) throws GcbPayException {
		String filePath = "D:\\public-rsa.cer";
		System.out.println(transform2PublicKeyString(filePath));

		String privateKeyPath = "D:\\private-rsa.pfx";
		System.out.println(transform2PrivateKeyString(privateKeyPath, "test-alias", "123456"));
	}
 
}
